==> neper-1.8.0.tgz: neper-1.8.0/src/ut/ut_string/ut_string.c <==
/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */
 
#include"ut_string.h"

int
ut_string_fnr (char *string, char find, char replace)
{
  int i, length, res;

  res = 0;
  length = strlen (string);
  for (i = 0; i <= length - 1; i++)
    if (string[i] == find)
    {
      string[i] = replace;
      res = 1;
    }

  return res;
}

int
ut_string_comp (char *in, char **list, int qty, char *out)
{
  int i;
  int res = 0;

  /* Checking if it matches exactly */
  for (i = 1; i <= qty; i++)	/* for every list parameter */
    if (strcmp (in, list[i]) == 0)
    {
      sprintf (out, "%s", list[i]);
      return 0;
    }

  for (i = 1; i <= qty; i++)	/* for every list parameter */
  {
    /* else, checking if one possibility could be ok */
    if (strncmp (in, list[i], strlen (in)) == 0)
    {
      if (res == 0)		/* this is a good possibility. */
	res = i;
      else			/* ie there are several possibilities: nok */
	return 1;
    }
  }

  if (res != 0)
  {
    sprintf (out, "%s", list[res]);	/* this is the goo possibility. */
    return 0;
  }
  else
    return -1;			/* there is no possibility. */
}

char *
ut_string_extension (char *in)
{
  char *ext;

  int i;
  int NoCin, NoCext;
  int DashPos = -1;

  NoCin = strlen (in);

  for (i = NoCin - 1; i >= 0; i--)
  {
    if (in[i] == '.')
    {
      DashPos = i;
      break;
    }
  }

  if (DashPos == -1)
    ext = NULL;
  else
  {
    NoCext = NoCin - DashPos;

    ext = ut_alloc_1d_char (NoCext + 1);

    for (i = 0; i <= NoCext - 1; i++)
      ext[i] = in[i + DashPos];
    ext[NoCext] = '\0';
  }

  return ext;
}

char *
ut_string_addextension (char *in, char *ext)
{
  char *out;

  unsigned int NoCin;		/* nb of characters of in */
  unsigned int NoCext;		/* nb of characters of extin */

  NoCin = strlen (in);
  NoCext = strlen (ext);

  out = ut_alloc_1d_char (NoCin + NoCext + 1);

  out = strncpy (out, in, NoCin);
  out = strcat (out, ext);
  out[NoCin + NoCext] = '\0';

  return out;
}



char *
ut_string_body (char *in)
{
  char *body;

  int i;
  int NoCin, NoCbody;
  int DashPos = -1;

  NoCin = strlen (in);

  for (i = NoCin - 1; i >= 0; i--)
  {
    if (in[i] == '.')
    {
      DashPos = i;
      break;
    }
  }

  if (DashPos == -1)
    NoCbody = NoCin;
  else
    NoCbody = DashPos;

  body = ut_alloc_1d_char (NoCbody + 1);

  for (i = 0; i <= NoCbody - 1; i++)
    body[i] = in[i];
  body[NoCbody] = '\0';

  return body;
}

int
ut_string_untilchar (char *string, char c, int *ppos, char *res)
{
  char *copy = ut_alloc_1d_char (strlen (string) + 1);
  /* char* copy ; */

  strcpy (copy, string + (*ppos));

  /* character `c' is replaced by `\0', which has for effect
   * to truncate the string -- it rests what we need.*/
  ut_string_fnr (copy, c, '\0');

  strcpy (res, copy);

  *ppos += strlen (copy);

  ut_free_1d_char (copy);

  if (strlen (res) > 0)
    return 1;
  else
    return 0;
  /*

     if ((*ppos) == (signed int)strlen (string))
     return 0;

     for (i = (*ppos); i < strlen (string) + 1; i++)
     if (string[i] != ',' && string[i] != '\0')
     {
     ext[j] = string[i];
     j++;
     }
     else
     break;

     (*ppos) = i + 1;
     res[j] = '\0';

     printf ("ext = %s\n", res);
   */
}


int
ut_string_thischarqty (char *string, char c)
{
  int i, res = 0;

  for (i = 0; i < (int) strlen (string); i++)
    if (string[i] == c)
      res++;

  return res;
}


int
ut_string_thischarpos (char *string, char c, int nb)
{
  int pos, tmp;

  /* by default, i.e. if nb == 0 or occurrence not found */
  pos = -1;

  if (nb != 0)
  {
    tmp = 0;
    for (pos = 0; pos < (int) strlen (string); pos++)
      if (string[pos] == c)
      {
	tmp++;
	if (tmp == nb)		/* the good occurence is found */
	  break;
      }
  }

  return pos;
}

void
ut_string_separate (char* string, char c, char*** parts, int* pqty)
{
  int i, pos;
  char** tmp = ut_alloc_2d_char (strlen(string), strlen(string));
  char*  val = ut_alloc_1d_char (strlen(string));
  
  (*pqty) = 0;
  pos = 0;
  while (ut_string_untilchar (string, c, &pos, val) == 1)
  {
    pos++;
    sprintf (tmp[(*pqty)++], "%s", val);
  }

  (*parts) = ut_alloc_1d_pchar (*pqty);
  for (i = 0; i < (*pqty); i++)
  {
    (*parts)[i] = ut_alloc_1d_char (strlen (tmp[i]));
    sprintf ((*parts)[i], "%s", tmp[i]);
  }

  ut_free_2d_char (tmp, strlen(string));
  ut_free_1d_char (val);

  return;
}

==> neper-1.8.1.tgz: neper-1.8.1/src/ut/ut_string/ut_string.c <==
/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */
 
#include"ut_string.h"
 
int
ut_string_fnr (char *string, char find, char replace)
{
  int i, length, res;

  res = 0;
  length = strlen (string);
  for (i = 0; i <= length - 1; i++)
    if (string[i] == find)
    {
      string[i] = replace;
      res = 1;
    }

  return res;
}

int
ut_string_fnrs (char *string, char* find, char* replace, int repeat)
{
  int i, j, p, equal, test = 0;
  int* pos = ut_alloc_1d_int (strlen (string) + 1);
  int length  = strlen (string);
  int lengthf = strlen (find);
  int lengthr = strlen (replace);
  char* string2 = ut_alloc_1d_char (ceil (length * lengthr / lengthf) + 10);

  while (repeat-- > 0)
  {
    length  = strlen (string);

    pos[0] = 0;

    /* looking for the pos of 'find' in 'string' */
    for (i = 0; i < length - lengthf + 1; i++)
    {
      equal = 1;
      for (j = 0; j < lengthf; j++)
      {
	if (string[i + j] != find[j])
	{
	  equal = 0;
	  break;
	}
      }

      if (equal == 1)
      {
	test = 1;
	pos[++pos[0]] = i;
	i += lengthf - 1;
      }
    }

    ut_board_1d_int_fprintf (stdout, pos, pos[0] + 1, "%d");

    if (pos[0] == 0)
      break;
    else
    {
      p = 0;
      for (i = 0; i < length; i++)
      {
	if (ut_board_1d_int_find (pos + 1, pos[0], i) == -1)
	  string2[p++] = string[i];
	else
	{
	  for (j = 0; j < lengthr; j++)
	    string2[p++] = replace[j];

	  i += lengthf - 1;
	}
      }
      string2[p]='\0';
    }
    sprintf (string, "%s", string2);
  }

  ut_free_1d_int (pos);
  ut_free_1d_char (string2);

  return test;
}

int
ut_string_comp (char *in, char **list, int qty, char *out)
{
  int i;
  int res = 0;

  /* Checking if it matches exactly */
  for (i = 1; i <= qty; i++)	/* for every list parameter */
    if (strcmp (in, list[i]) == 0)
    {
      sprintf (out, "%s", list[i]);
      return 0;
    }

  for (i = 1; i <= qty; i++)	/* for every list parameter */
  {
    /* else, checking if one possibility could be ok */
    if (strncmp (in, list[i], strlen (in)) == 0)
    {
      if (res == 0)		/* this is a good possibility. */
	res = i;
      else			/* ie there are several possibilities: nok */
	return 1;
    }
  }

  if (res != 0)
  {
    sprintf (out, "%s", list[res]);	/* this is the goo possibility. */
    return 0;
  }
  else
    return -1;			/* there is no possibility. */
}

char *
ut_string_extension (char *in)
{
  char *ext;

  int i;
  int NoCin, NoCext;
  int DashPos = -1;

  NoCin = strlen (in);

  for (i = NoCin - 1; i >= 0; i--)
  {
    if (in[i] == '.')
    {
      DashPos = i;
      break;
    }
  }

  if (DashPos == -1)
    ext = NULL;
  else
  {
    NoCext = NoCin - DashPos;

    ext = ut_alloc_1d_char (NoCext + 1);

    for (i = 0; i <= NoCext - 1; i++)
      ext[i] = in[i + DashPos];
    ext[NoCext] = '\0';
  }

  return ext;
}

char *
ut_string_addextension (char *in, char *ext)
{
  char *out;

  unsigned int NoCin;		/* nb of characters of in */
  unsigned int NoCext;		/* nb of characters of extin */
  int skip = 0;

  NoCin = strlen (in);
  NoCext = strlen (ext);

  if (ext[0] == '.')
  {
    NoCext--;
    skip++;
  }

  out = ut_alloc_1d_char (NoCin + NoCext + 2);

  /* out = strncpy (out, in, NoCin);
  out = strcat (out, ext + 1); */
  sprintf (out, "%s.%s", in, ext + skip);
  /* out[NoCin + NoCext + skip] = '\0'; */

  return out;
}

char *
ut_string_body (char *in)
{
  char *body;

  int i;
  int NoCin, NoCbody;
  int DashPos = -1;

  NoCin = strlen (in);

  for (i = NoCin - 1; i >= 0; i--)
  {
    if (in[i] == '.')
    {
      DashPos = i;
      break;
    }
  }

  if (DashPos == -1)
    NoCbody = NoCin;
  else
    NoCbody = DashPos;

  body = ut_alloc_1d_char (NoCbody + 1);

  for (i = 0; i <= NoCbody - 1; i++)
    body[i] = in[i];
  body[NoCbody] = '\0';

  return body;
}

char*
ut_string_changeextension (char * string, char * newext)
{
  char* body = ut_alloc_1d_char (strlen (string));
  char* res;

  body = ut_string_body (string);
  res = ut_string_addextension (body, newext);

  ut_free_1d_char (body);
  
  return res;
}

int
ut_string_untilchar (char *string, char c, int *ppos, char *res)
{
  char *copy = ut_alloc_1d_char (strlen (string) + 1);

  strcpy (copy, string + (*ppos));

  ut_string_fnr (copy, c, '\0');

  strcpy (res, copy);

  *ppos += strlen (copy);

  ut_free_1d_char (copy);

  if (strlen (res) > 0)
    return 1;
  else
    return 0;
  /*

     if ((*ppos) == (signed int)strlen (string))
     return 0;

     for (i = (*ppos); i < strlen (string) + 1; i++)
     if (string[i] != ',' && string[i] != '\0')
     {
     ext[j] = string[i];
     j++;
     }
     else
     break;

     (*ppos) = i + 1;
     res[j] = '\0';

     printf ("ext = %s\n", res);
   */
}

int
ut_string_thischarqty (char *string, char c)
{
  int i, res = 0;

  for (i = 0; i < (int) strlen (string); i++)
    if (string[i] == c)
      res++;

  return res;
}

int
ut_string_thischarpos (char *string, char c, int nb)
{
  int pos, tmp;

  /* by default, i.e. if nb == 0 or occurrence not found */
  pos = -1;

  if (nb != 0)
  {
    tmp = 0;
    for (pos = 0; pos < (int) strlen (string); pos++)
      if (string[pos] == c)
      {
	tmp++;
	if (tmp == nb)		/* the good occurence is found */
	  break;
      }
  }

  return pos;
}

void
ut_string_separate (char* string, char c, char*** parts, int* pqty)
{
  int i, pos;
  char** tmp = ut_alloc_2d_char (strlen(string), strlen(string));
  char*  val = ut_alloc_1d_char (strlen(string) + 1);
  
  (*pqty) = 0;
  pos = 0;
  while (ut_string_untilchar (string, c, &pos, val))
  {
    pos++;
    sprintf (tmp[(*pqty)++], "%s", val);
  }

  (*parts) = ut_alloc_1d_pchar (*pqty);
  for (i = 0; i < (*pqty); i++)
  {
    (*parts)[i] = ut_alloc_1d_char (strlen (tmp[i]));
    sprintf ((*parts)[i], "%s", tmp[i]);
  }

  ut_free_2d_char (tmp, strlen(string));
  ut_free_1d_char (val);

  return;
}

void
ut_string_partqty (char* string, char c, int* pqty)
{
  int pos;
  char*  val = ut_alloc_1d_char (strlen(string));
  
  (*pqty) = 0;
  pos = 0;
  while (ut_string_untilchar (string, c, &pos, val) == 1)
  {
    (*pqty)++;
    pos++;
  }

  ut_free_1d_char (val);

  return;
}

int
ut_string_part (char* string, char c, int id, char* part)
{
  int pos, nb, res;
  char* val = ut_alloc_1d_char (strlen(string) + 1);
  
  res = 0;
  nb = 0;
  pos = 0;
  while (ut_string_untilchar (string, c, &pos, val) == 1)
  {
    pos++;
    nb++;
    if (nb == id)
    {
      res = 1;
      break;
    }
  }

  sprintf (part, "%s", val);

  ut_free_1d_char (val);

  return res;
}

/* could be optimized: do not break the list */
int
ut_string_inlist (char* string, char c, char* part)
{
  char* tmp = ut_alloc_1d_char (strlen (string) + 3);

  sprintf (tmp, "%c%s%c", c, string, c);

  return (strstr (tmp, part) != NULL) ? 1 : 0;
}

int
ut_string_nbwords (char* string)
{
  unsigned int i;
  int bef, nbw;

  nbw = 0;
  bef = 0;
  for (i = 0; i < strlen (string); i++)
  {
    if (string[i] != ' ' && string[i] != '\t' && string[i] != '\n')
    {
      if (bef == 0)
      {
	nbw++;
	bef = 1;
      }
    }
    else
      bef = 0;
  }

  return nbw;
}
